@@ -1,8 +1,11 @@
+#![feature(array_chunks)]
+
+pub mod zest_block;
 pub mod zest_dec;
 pub mod zest_digest;
 pub mod zest_enc;
 pub mod zest_hash;
 pub mod zest_rand;
 // Internal usage
-mod counter;
+mod num_def;
 mod zest_pad;
