@@ -2,66 +2,47 @@ use std::{
     intrinsics::transmute,
     io::{Read, Result},
     mem::{size_of, zeroed},
+    ops::{Add, Mul, Rem, Sub},
     ptr::addr_of,
 };
 
-use num::{Bounded, Integer};
-
-use crate::{counter::inc_counter, zest_hash::zest4096};
+use crate::{num_def::NumberSupport, zest_block::ZestBlock, zest_hash::zest4096};
 
 // Notice: The seed of the PRNG also holds the counter, in the higher array indicies.
 // Keep your key/seed in the low bits, and the nonce after that if you wish.
 // The top 16 bytes for the CSPRNG will be "Bytes are fun :)" in big endian,
 // The next highest bytes will be counted over once the current 512 byte buffer
 // has been read from.
-
-const MAGIC_BYTES: [u64; 2] = [
-    u64::from_be_bytes(*b"Bytes ar"),
-    u64::from_be_bytes(*b"e fun :)"),
-];
-
 #[derive(Clone, Copy)]
 pub struct ZestRand {
     // Seed of the CSPRNG (THIS INCLUDES THE COUNTER) -- higher bits get changed first
-    seed: [u64; 62],
+    seed: ZestBlock,
     // Current output of the CSPRNG
-    hash_output: [u64; 64],
+    hash_output: ZestBlock,
     // Current byte of the CSPRNG (in the output)
     byte_offset: usize,
 }
 impl ZestRand {
-    // (Re)seed the CSPRNG, and start at beginning of block + offset
+    // (Re)seed the CSPRNG, and start at beginning of block + offset.
+    // Doesn't check for identical seeds before rehashing.
     #[inline(always)]
-    pub fn seed(&mut self, seed: [u64; 62], byte_offset: usize) {
-        if self.seed != seed {
-            self.seed = seed;
-            self.hash_output = zest4096(self.generate_input());
-        }
+    pub fn seed(&mut self, seed: ZestBlock, byte_offset: usize) {
+        self.seed = seed;
+        self.hash_output = zest4096(self.seed);
         self.byte_offset = byte_offset & 0x1FF;
     }
     // Counts to the next block, (lower index ptr = higher place value)
     // and sets the hash output to None
     #[inline(always)]
     pub fn next_block(&mut self) {
-        inc_counter(&mut self.seed);
-        self.hash_output = zest4096(self.generate_input());
+        self.seed.increment_block();
+        self.hash_output = zest4096(self.seed);
     }
     // Get the current block
     #[inline(always)]
-    pub fn get_block(&mut self) -> [u64; 64] {
+    pub fn get_block(&mut self) -> ZestBlock {
         self.hash_output
     }
-    // Generate the input to the zest hash based on seed, index, and magic bytes
-    #[inline(always)]
-    fn generate_input(&self) -> [u64; 64] {
-        let mut block = [0_u64; 64];
-        let (seed_split, magic_split) = block.split_at_mut(62);
-        // Copy the seed to the first 62 64 bit blocks
-        seed_split.copy_from_slice(&self.seed);
-        // Copy the magic bytes to the last 2 64 bit blocks
-        magic_split.copy_from_slice(&MAGIC_BYTES);
-        block
-    }
 }
 
 // Functions which output random data:
@@ -103,7 +84,7 @@ impl ZestRand {
     }
     // Use to generate random integers.
     #[inline(always)]
-    pub fn next_num<T: Integer + Copy>(&mut self) -> T {
+    pub fn next_num<T: NumberSupport + Copy>(&mut self) -> T {
         unsafe { self.fill_rand() }
     }
     // Function takes in any signed integer type, and the max and minimum value it could be.
@@ -111,19 +92,27 @@ impl ZestRand {
     // The maximum range is **not** checked to be the actual highest of the two arguments,
     // the maximum range is also ***non-inclusive***.
     #[inline(always)]
-    pub fn bounded<A: Integer + Copy + From<u8> + Bounded>(&mut self, min: A, max: A) -> A {
-        let max_val: A = Bounded::max_value();
+    pub fn bounded<
+        A: NumberSupport
+            + Copy
+            + Sub<Output = A>
+            + Add<Output = A>
+            + Mul<Output = A>
+            + Rem<Output = A>
+            + PartialOrd
+            + From<u8>,
+    >(
+        &mut self,
+        min: A,
+        max: A,
+    ) -> A {
+        let max_val: A = A::MAX;
         let mut not_biased = self.next_num::<A>();
         let range = max - min;
         while not_biased > (max_val - (((max_val % range) + A::from(1)) % range)) {
             not_biased = self.next_num::<A>();
         }
-        if not_biased < A::from(0) {
-            // Calculate absolute value to make certain
-            // the numbers are not symetrical over the positive/negatice range
-            not_biased = not_biased * (A::from(0) - A::from(1));
-        }
-        not_biased % range + min
+        not_biased.abs() % range + min
     }
 }
 impl Default for ZestRand {
@@ -133,7 +122,7 @@ impl Default for ZestRand {
         // are pointers to any other object.
         // The initial block is also generated.
         let mut initial: Self = unsafe { zeroed() };
-        initial.hash_output = zest4096(initial.generate_input());
+        initial.hash_output = zest4096(initial.seed);
         initial
     }
 }
@@ -190,7 +179,7 @@ impl ZestRand {
     // Fill a vector with random numbers.
     // Wrapper around fill_vec_any.
     #[inline(always)]
-    pub fn fill_vec_num<T: Sized + Copy + Integer>(&mut self, len: usize) -> Vec<T> {
+    pub fn fill_vec_num<T: NumberSupport + Copy>(&mut self, len: usize) -> Vec<T> {
         unsafe { self.fill_vec_any(len) }
     }
 }
